package mapper

import (
	"fmt"
	"strings"

	"github.com/rancher/norman/pkg/data"
	"github.com/rancher/norman/pkg/types"
	"github.com/rancher/norman/pkg/types/convert"
	"github.com/rancher/wrangler/pkg/kv"
)

type Enum struct {
	DefaultMapper
	field string
	vals  map[string]string
}

func NewEnum(field string, vals ...string) types.Mapper {
	f := &Enum{
		DefaultMapper: DefaultMapper{
			Field: field,
		},
		vals: map[string]string{},
	}

	for _, v := range vals {
		k := v
		if strings.Contains(v, "=") {
			v, k = kv.Split(v, "=")
		}
		f.vals[normalize(v)] = k
	}

	return f
}

func normalize(v string) string {
	v = strings.ReplaceAll(v, "_", "")
	v = strings.ReplaceAll(v, "-", "")
	return strings.ToLower(v)
}

func (d *Enum) FromInternal(data data.Object) {
}

func (d *Enum) ToInternal(data data.Object) error {
	v, ok := data[d.Field]
	if ok {
		newValue, ok := d.vals[normalize(convert.ToString(v))]
		if !ok {
			return fmt.Errorf("%s is not a valid value for field %s", v, d.Field)
		}
		data[d.Field] = newValue
	}
	return nil
}
