/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/solo-io/gloo/projects/gloo/pkg/api/v1/kube/apis/gloo.solo.io/v1"
	scheme "github.com/solo-io/gloo/projects/gloo/pkg/api/v1/kube/client/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// ArtifactsGetter has a method to return a ArtifactInterface.
// A group's client should implement this interface.
type ArtifactsGetter interface {
	Artifacts(namespace string) ArtifactInterface
}

// ArtifactInterface has methods to work with Artifact resources.
type ArtifactInterface interface {
	Create(*v1.Artifact) (*v1.Artifact, error)
	Update(*v1.Artifact) (*v1.Artifact, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.Artifact, error)
	List(opts metav1.ListOptions) (*v1.ArtifactList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Artifact, err error)
	ArtifactExpansion
}

// artifacts implements ArtifactInterface
type artifacts struct {
	client rest.Interface
	ns     string
}

// newArtifacts returns a Artifacts
func newArtifacts(c *GlooV1Client, namespace string) *artifacts {
	return &artifacts{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the artifact, and returns the corresponding artifact object, and an error if there is any.
func (c *artifacts) Get(name string, options metav1.GetOptions) (result *v1.Artifact, err error) {
	result = &v1.Artifact{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("artifacts").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Artifacts that match those selectors.
func (c *artifacts) List(opts metav1.ListOptions) (result *v1.ArtifactList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.ArtifactList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("artifacts").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested artifacts.
func (c *artifacts) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("artifacts").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a artifact and creates it.  Returns the server's representation of the artifact, and an error, if there is any.
func (c *artifacts) Create(artifact *v1.Artifact) (result *v1.Artifact, err error) {
	result = &v1.Artifact{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("artifacts").
		Body(artifact).
		Do().
		Into(result)
	return
}

// Update takes the representation of a artifact and updates it. Returns the server's representation of the artifact, and an error, if there is any.
func (c *artifacts) Update(artifact *v1.Artifact) (result *v1.Artifact, err error) {
	result = &v1.Artifact{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("artifacts").
		Name(artifact.Name).
		Body(artifact).
		Do().
		Into(result)
	return
}

// Delete takes name of the artifact and deletes it. Returns an error if one occurs.
func (c *artifacts) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("artifacts").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *artifacts) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("artifacts").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched artifact.
func (c *artifacts) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Artifact, err error) {
	result = &v1.Artifact{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("artifacts").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
